# Autogenerated by Astropy-affiliated package astropy_helpers's setup.py on 2016-05-09 15:26:59.132943
from __future__ import unicode_literals
import datetime

version = "1.1.2"
githash = "d8f48901442e4056879c4249e73ecd7d04a28282"


major = 1
minor = 1
bugfix = 2

release = True
timestamp = datetime.datetime(2016, 5, 9, 15, 26, 59, 132943)
debug = False

try:
    from ._compiler import compiler
except ImportError:
    compiler = "unknown"

try:
    from .cython_version import cython_version
except ImportError:
    cython_version = "unknown"
